@@ -11,7 +11,7 @@ import numpy as np
 from sparsetools import coo_tocsr, coo_todense, coo_matvec
 from base import isspmatrix
 from data import _data_matrix
-from sputils import upcast, to_native, isshape, getdtype, isintlike
+from sputils import upcast, upcast_char, to_native, isshape, getdtype, isintlike
 
 class coo_matrix(_data_matrix):
     """
@@ -30,19 +30,38 @@ class coo_matrix(_data_matrix):
             to construct an empty matrix with shape (M, N)
             dtype is optional, defaulting to dtype='d'.
 
-        coo_matrix((data, ij), [shape=(M, N)])
-            The arguments 'data' and 'ij' represent three arrays:
+        coo_matrix((data, (i, j)), [shape=(M, N)])
+            to construct from three arrays:
                 1. data[:]   the entries of the matrix, in any order
-                2. ij[0][:]  the row indices of the matrix entries
-                3. ij[1][:]  the column indices of the matrix entries
+                2. i[:]      the row indices of the matrix entries
+                3. j[:]      the column indices of the matrix entries
 
-            Where ``A[ij[0][k], ij[1][k] = data[k]``.  When shape is
-            not specified, it is inferred from the index arrays
+            Where ``A[i[k], j[k]] = data[k]``.  When shape is not
+            specified, it is inferred from the index arrays
 
+    Attributes
+    ----------
+    dtype : dtype
+        Data type of the matrix
+    shape : 2-tuple
+        Shape of the matrix
+    ndim : int
+        Number of dimensions (this is always 2)
+    nnz
+        Number of nonzero elements
+    data
+        COO format data array of the matrix
+    row
+        COO format row index array of the matrix
+    col
+        COO format column index array of the matrix
 
     Notes
     -----
 
+    Sparse matrices can be used in arithmetic operations: they support
+    addition, subtraction, multiplication, division, and matrix power.
+
     Advantages of the COO format
         - facilitates fast conversion among sparse formats
         - permits duplicate entries (see example)
@@ -61,38 +80,34 @@ class coo_matrix(_data_matrix):
           entries will be summed together.  This facilitates efficient
           construction of finite element matrices and the like. (see example)
 
-
     Examples
     --------
-
-    >>> from scipy.sparse import *
-    >>> from scipy import *
-    >>> coo_matrix( (3,4), dtype=int8 ).todense()
+    >>> from scipy.sparse import coo_matrix
+    >>> coo_matrix((3,4), dtype=np.int8).todense()
     matrix([[0, 0, 0, 0],
             [0, 0, 0, 0],
             [0, 0, 0, 0]], dtype=int8)
 
-    >>> row  = array([0,3,1,0])
-    >>> col  = array([0,3,1,2])
-    >>> data = array([4,5,7,9])
-    >>> coo_matrix( (data,(row,col)), shape=(4,4) ).todense()
+    >>> row  = np.array([0,3,1,0])
+    >>> col  = np.array([0,3,1,2])
+    >>> data = np.array([4,5,7,9])
+    >>> coo_matrix((data,(row,col)), shape=(4,4)).todense()
     matrix([[4, 0, 9, 0],
             [0, 7, 0, 0],
             [0, 0, 0, 0],
             [0, 0, 0, 5]])
 
     >>> # example with duplicates
-    >>> row  = array([0,0,1,3,1,0,0])
-    >>> col  = array([0,2,1,3,1,0,0])
-    >>> data = array([1,1,1,1,1,1,1])
-    >>> coo_matrix( (data,(row,col)), shape=(4,4)).todense()
+    >>> row  = np.array([0,0,1,3,1,0,0])
+    >>> col  = np.array([0,2,1,3,1,0,0])
+    >>> data = np.array([1,1,1,1,1,1,1])
+    >>> coo_matrix((data, (row,col)), shape=(4,4)).todense()
     matrix([[3, 0, 1, 0],
             [0, 2, 0, 0],
             [0, 0, 0, 0],
             [0, 0, 0, 1]])
 
     """
-
     def __init__(self, arg1, shape=None, dtype=None, copy=False):
         _data_matrix.__init__(self)
 
@@ -162,14 +177,14 @@ class coo_matrix(_data_matrix):
 
                 if np.rank(M) != 2:
                     raise TypeError('expected rank <= 2 array or matrix')
+
                 self.shape = M.shape
-                self.row,self.col = (M != 0).nonzero()
-                self.data  = M[self.row,self.col]
+                self.row, self.col = M.nonzero()
+                self.data  = M[self.row, self.col]
 
         if dtype is not None:
             self.data = self.data.astype(dtype)
 
-
         self._check()
 
     def getnnz(self):
@@ -215,10 +230,15 @@ class coo_matrix(_data_matrix):
         M,N = self.shape
         return coo_matrix((self.data, (self.col, self.row)), shape=(N,M), copy=copy)
 
-    def toarray(self):
-        B = np.zeros(self.shape, dtype=self.dtype)
+    def toarray(self, order=None, out=None):
+        """See the docstring for `spmatrix.toarray`."""
+        B = self._process_toarray_args(order, out)
+        fortran = int(B.flags.f_contiguous)
+        if not fortran and not B.flags.c_contiguous:
+            raise ValueError("Output array must be C or F contiguous")
         M,N = self.shape
-        coo_todense(M, N, self.nnz, self.row, self.col, self.data, B.ravel())
+        coo_todense(M, N, self.nnz, self.row, self.col, self.data,
+                    B.ravel('A'), fortran)
         return B
 
     def tocsc(self):
@@ -226,8 +246,8 @@ class coo_matrix(_data_matrix):
 
         Duplicate entries will be summed together.
 
-        Example
-        -------
+        Examples
+        --------
         >>> from numpy import array
         >>> from scipy.sparse import coo_matrix
         >>> row  = array([0,0,1,3,1,0,0])
@@ -264,8 +284,8 @@ class coo_matrix(_data_matrix):
 
         Duplicate entries will be summed together.
 
-        Example
-        -------
+        Examples
+        --------
         >>> from numpy import array
         >>> from scipy.sparse import coo_matrix
         >>> row  = array([0,0,1,3,1,0,0])
@@ -350,14 +370,14 @@ class coo_matrix(_data_matrix):
 
     def _mul_vector(self, other):
         #output array
-        result = np.zeros( self.shape[0], dtype=upcast(self.dtype,other.dtype) )
+        result = np.zeros( self.shape[0], dtype=upcast_char(self.dtype.char,
+                                                            other.dtype.char) )
         coo_matvec(self.nnz, self.row, self.col, self.data, other, result)
         return result
 
     def _mul_multivector(self, other):
         return np.hstack( [ self._mul_vector(col).reshape(-1,1) for col in other.T ] )
 
-from sputils import _isinstance
 
 def isspmatrix_coo( x ):
-    return _isinstance(x, coo_matrix)
+    return isinstance(x, coo_matrix)
