@@ -13,15 +13,18 @@ Run tests if sparse is not installed:
   python tests/test_sparse.py
 """
 
+import sys
 import warnings
 
 import numpy as np
 from numpy import arange, zeros, array, dot, matrix, asmatrix, asarray, \
                   vstack, ndarray, transpose, diag, kron, inf, conjugate, \
-                  int8
+                  int8, ComplexWarning
 
 import random
-from numpy.testing import *
+from numpy.testing import assert_raises, assert_equal, assert_array_equal, \
+        assert_array_almost_equal, assert_almost_equal, assert_, \
+        dec, TestCase, run_module_suite
 
 import scipy.sparse as sparse
 from scipy.sparse import csc_matrix, csr_matrix, dok_matrix, \
@@ -32,6 +35,7 @@ from scipy.sparse.linalg import splu
 
 
 warnings.simplefilter('ignore', SparseEfficiencyWarning)
+warnings.simplefilter('ignore', ComplexWarning)
 
 
 #TODO check that spmatrix( ... , copy=X ) is respected
@@ -65,7 +69,9 @@ class _TestCommon:
         """Test manipulating empty matrices. Fails in SciPy SVN <= r1768
         """
         shape = (5, 5)
-        for mytype in ['int32', 'float32', 'float64', 'complex64', 'complex128']:
+        for mytype in [np.dtype('int32'), np.dtype('float32'),
+                np.dtype('float64'), np.dtype('complex64'),
+                np.dtype('complex128')]:
             a = self.spmatrix(shape, dtype=mytype)
             b = a + a
             c = 2 * a
@@ -128,9 +134,11 @@ class _TestCommon:
 
     def test_getrow(self):
         assert_array_equal(self.datsp.getrow(1).todense(), self.dat[1,:])
+        assert_array_equal(self.datsp.getrow(-1).todense(), self.dat[-1,:])
 
     def test_getcol(self):
         assert_array_equal(self.datsp.getcol(1).todense(), self.dat[:,1])
+        assert_array_equal(self.datsp.getcol(-1).todense(), self.dat[:,-1])
 
     def test_sum(self):
         """Does the matrix's .sum(axis=...) method work?
@@ -201,8 +209,33 @@ class _TestCommon:
     #    assert_equal( array(self.datsp), self.dat )
 
     def test_todense(self):
+        # Check C-contiguous (default).
         chk = self.datsp.todense()
-        assert_array_equal(chk,self.dat)
+        assert_array_equal(chk, self.dat)
+        assert_(chk.flags.c_contiguous)
+        assert_(not chk.flags.f_contiguous)
+        # Check C-contiguous (with arg).
+        chk = self.datsp.todense(order='C')
+        assert_array_equal(chk, self.dat)
+        assert_(chk.flags.c_contiguous)
+        assert_(not chk.flags.f_contiguous)
+        # Check F-contiguous (with arg).
+        chk = self.datsp.todense(order='F')
+        assert_array_equal(chk, self.dat)
+        assert_(not chk.flags.c_contiguous)
+        assert_(chk.flags.f_contiguous)
+        # Check with out argument (array).
+        out = np.zeros(self.datsp.shape, dtype=self.datsp.dtype)
+        chk = self.datsp.todense(out=out)
+        assert_array_equal(self.dat, out)
+        assert_array_equal(self.dat, chk)
+        assert_(chk.base is out)
+        # Check with out array (matrix).
+        out = np.asmatrix(np.zeros(self.datsp.shape, dtype=self.datsp.dtype))
+        chk = self.datsp.todense(out=out)
+        assert_array_equal(self.dat, out)
+        assert_array_equal(self.dat, chk)
+        assert_(chk is out)
         a = matrix([1.,2.,3.])
         dense_dot_dense = a * self.dat
         check = a * self.datsp.todense()
@@ -213,9 +246,30 @@ class _TestCommon:
         assert_array_equal(dense_dot_dense, check2)
 
     def test_toarray(self):
+        # Check C-contiguous (default).
         dat = asarray(self.dat)
         chk = self.datsp.toarray()
         assert_array_equal(chk, dat)
+        assert_(chk.flags.c_contiguous)
+        assert_(not chk.flags.f_contiguous)
+        # Check C-contiguous (with arg).
+        chk = self.datsp.toarray(order='C')
+        assert_array_equal(chk, dat)
+        assert_(chk.flags.c_contiguous)
+        assert_(not chk.flags.f_contiguous)
+        # Check F-contiguous (with arg).
+        chk = self.datsp.toarray(order='F')
+        assert_array_equal(chk, dat)
+        assert_(not chk.flags.c_contiguous)
+        assert_(chk.flags.f_contiguous)
+        # Check with output arg.
+        out = np.zeros(self.datsp.shape, dtype=self.datsp.dtype)
+        self.datsp.toarray(out=out)
+        assert_array_equal(chk, dat)
+        # Check that things are fine when we don't initialize with zeros.
+        out[...] = 1.
+        self.datsp.toarray(out=out)
+        assert_array_equal(chk, dat)
         a = array([1.,2.,3.])
         dense_dot_dense = dot(a, dat)
         check = dot(a, self.datsp.toarray())
@@ -239,15 +293,15 @@ class _TestCommon:
     def test_asfptype(self):
         A = self.spmatrix( arange(6,dtype='int32').reshape(2,3) )
 
-        assert_equal( A.dtype , 'int32' )
-        assert_equal( A.asfptype().dtype, 'float64' )
+        assert_equal( A.dtype , np.dtype('int32') )
+        assert_equal( A.asfptype().dtype, np.dtype('float64') )
         assert_equal( A.asfptype().format, A.format )
-        assert_equal( A.astype('int16').asfptype().dtype , 'float32' )
-        assert_equal( A.astype('complex128').asfptype().dtype , 'complex128' )
+        assert_equal( A.astype('int16').asfptype().dtype , np.dtype('float32') )
+        assert_equal( A.astype('complex128').asfptype().dtype , np.dtype('complex128') )
 
         B = A.asfptype()
         C = B.asfptype()
-        assert( B is C )
+        assert_( B is C )
 
 
     def test_mul_scalar(self):
@@ -289,6 +343,14 @@ class _TestCommon:
         assert_array_equal(A.todense() - self.datsp,A.todense() - self.dat)
         assert_array_equal(self.datsp - A.todense(),self.dat - A.todense())
 
+    def test_add0(self):
+        """ Adding 0 to a sparse matrix """
+        assert_array_equal((self.datsp + 0).todense(), self.dat)
+        # use sum (which takes 0 as a starting value)
+        sumS = sum([k * self.datsp for k in range(1, 3)])
+        sumD = sum([k * self.dat for k in range(1, 3)])
+        assert_almost_equal(sumS.todense(), sumD)
+
     def test_elementwise_multiply(self):
         # real/real
         A = array([[4,0,9],[2,-3,5]])
@@ -353,7 +415,7 @@ class _TestCommon:
         """
         A = self.spmatrix([[1],[2],[3]])
 
-        assert(isspmatrix(A * array(1)))
+        assert_(isspmatrix(A * array(1)))
         assert_equal((A * array(1)).todense(), [[1],[2],[3]])
         assert_equal(A * array([1]), array([1,2,3]))
         assert_equal(A * array([[1]]), array([[1],[2],[3]]))
@@ -369,8 +431,8 @@ class _TestCommon:
         assert_equal((M * matrix([[1],[2],[3]])).shape,(4,1))
 
         #check result type
-        assert(isinstance( M * array([1,2,3]), ndarray))
-        assert(isinstance( M * matrix([1,2,3]).T, matrix))
+        assert_(isinstance( M * array([1,2,3]), ndarray))
+        assert_(isinstance( M * matrix([1,2,3]).T, matrix))
 
         #ensure exception is raised for improper dimensions
         bad_vecs = [array([1,2]), array([1,2,3,4]), array([[1],[2]]),
@@ -461,7 +523,7 @@ class _TestCommon:
 
         for b in bs:
             result = asp*b
-            assert( isinstance(result, type(b)) )
+            assert_( isinstance(result, type(b)) )
             assert_equal( result.shape, (4,2) )
             assert_equal( result, dot(a,b) )
 
@@ -548,8 +610,18 @@ class _TestCommon:
         assert_equal(toself(copy=False).todense(), A.todense())
 
 
-        #TODO how can we check whether the data is copied?
-        pass
+        # check whether the data is copied?
+        # TODO: deal with non-indexable types somehow
+        B = A.copy()
+        try:
+            B[0,0] += 1
+            assert_(B[0,0] != A[0,0])
+        except NotImplementedError:
+            # not all sparse matrices can be indexed
+            pass
+        except TypeError:
+            # not all sparse matrices can be indexed
+            pass
 
     # Eventually we'd like to allow matrix products between dense
     # and sparse matrices using the normal dot() function:
@@ -566,7 +638,6 @@ class _TestCommon:
     #    assert_array_equal(dense_dot_dense, dense_dot_sparse)
 
 
-
 class _TestInplaceArithmetic:
     def test_imul_scalar(self):
         a = self.datsp.copy()
@@ -590,6 +661,7 @@ class _TestInplaceArithmetic:
 class _TestGetSet:
     def test_setelement(self):
         A = self.spmatrix((3,4))
+        A[ 0, 0] = 0 # bug 870
         A[ 1, 2] = 4.0
         A[ 0, 1] = 3
         A[ 2, 0] = 2.0
@@ -601,6 +673,12 @@ class _TestGetSet:
         for ij in [(0,4),(-1,4),(3,0),(3,4),(3,-1)]:
             assert_raises(IndexError, A.__setitem__, ij, 123.0)
 
+        for v in [[1,2,3], array([1,2,3])]:
+            assert_raises(ValueError, A.__setitem__, (0,0), v)
+
+        for v in [3j]:
+            assert_raises(TypeError, A.__setitem__, (0,0), v)
+
     def test_getelement(self):
         D = array([[1,0,0],
                    [4,3,0],
@@ -669,7 +747,7 @@ class _TestHorizSlicing:
             a = A[6,3:7]
         except IndexError:
             caught += 1
-        assert caught == 2
+        assert_(caught == 2)
 
 
 class _TestVertSlicing:
@@ -704,8 +782,7 @@ class _TestVertSlicing:
             a = A[6,3:7]
         except IndexError:
             caught += 1
-        assert caught == 2
-
+        assert_(caught == 2)
 
 
 class _TestBothSlicing:
@@ -726,10 +803,28 @@ class _TestBothSlicing:
         assert_array_equal(E[1:2, 1:2], F[1:2, 1:2].todense())
         assert_array_equal(E[:, 1:], F[:, 1:].todense())
 
+
 class _TestFancyIndexing:
     """Tests fancy indexing features.  The tests for any matrix formats
     that implement these features should derive from this class.
     """
+    def test_fancy_indexing_set(self):
+        n, m = (5, 10)
+        def _test_set(i, j, nitems):
+            A = self.spmatrix((n, m))
+            A[i, j] = 1
+            assert_almost_equal(A.sum(), nitems)
+            assert_almost_equal(A[i, j], 1)
+
+        # [i,j]
+        for i, j in [(2, 3), (-1, 8), (-1, -2), (array(-1), -2), (-1, array(-2)),
+                     (array(-1), array(-2))]:
+            _test_set(i, j, 1)
+
+        # [i,1:2]
+        for i, j in [(2, slice(m)), (2, slice(5, -2)), (array(2), slice(5, -2))]:
+            _test_set(i, j, 3)
+
     def test_fancy_indexing(self):
         B = asmatrix(arange(50).reshape(5,10))
         A = self.spmatrix( B )
@@ -830,6 +925,30 @@ class _TestFancyIndexing:
         assert_equal(A[s,:].todense(), B[2:4,:])
         assert_equal(A[:,s].todense(), B[:,2:4])
 
+    def test_fancy_indexing_randomized(self):
+        random.seed(0) # make runs repeatable
+
+        NUM_SAMPLES = 50
+        M = 6
+        N = 4
+
+        D = np.asmatrix(np.random.rand(M,N))
+        D = np.multiply(D, D > 0.5)
+
+        I = np.random.random_integers(-M + 1, M - 1, size=NUM_SAMPLES)
+        J = np.random.random_integers(-N + 1, N - 1, size=NUM_SAMPLES)
+
+        S = self.spmatrix(D)
+
+        assert_equal(S[I,J], D[I,J])
+
+        I_bad = I + M
+        J_bad = J - N
+
+        assert_raises(IndexError, S.__getitem__, (I_bad,J))
+        assert_raises(IndexError, S.__getitem__, (I,J_bad))
+
+
 class _TestArithmetic:
     """
     Test real/complex arithmetic
@@ -910,13 +1029,27 @@ class _TestArithmetic:
                 assert_equal(S1.dtype,D1.dtype)
 
 
+class _Test2DSlicingRegression:
+    def test_non_unit_stride_2d_indexing_raises_exception(self):
+        # Regression test -- used to silently ignore the stride.
+        try:
+            self.spmatrix((500, 500))[0:100:2, 0:100:2]
+        except ValueError:
+            return
+        assert_(False)  # Should not happen.
+
 
 class TestCSR(_TestCommon, _TestGetSet, _TestSolve,
         _TestInplaceArithmetic, _TestArithmetic,
         _TestHorizSlicing, _TestVertSlicing, _TestBothSlicing,
-        _TestFancyIndexing, TestCase):
+        _TestFancyIndexing, _Test2DSlicingRegression, TestCase):
     spmatrix = csr_matrix
 
+    @dec.knownfailureif(True, "Fancy indexing is known to be broken for CSR" \
+                              " matrices")
+    def test_fancy_indexing_set(self):
+        _TestFancyIndexing.test_fancy_indexing_set(self)
+
     def test_constructor1(self):
         b = matrix([[0,4,0],
                    [3,0,0],
@@ -984,7 +1117,6 @@ class TestCSR(_TestCommon, _TestGetSet, _TestSolve,
         csr = csr_matrix((data, indices, indptr))
         assert_array_equal(csr.shape,(3,6))
 
-
     def test_sort_indices(self):
         data    = arange( 5 )
         indices = array( [7, 2, 1, 5, 4] )
@@ -1006,14 +1138,41 @@ class TestCSR(_TestCommon, _TestGetSet, _TestSolve,
         assert_array_equal(asp.data,[1, 2, 3])
         assert_array_equal(asp.todense(),bsp.todense())
 
+    def test_ufuncs(self):
+        X = csr_matrix(np.arange(20).reshape(4, 5) / 20.)
+        for f in ["sin", "tan", "arcsin", "arctan", "sinh", "tanh",
+                  "arcsinh", "arctanh", "rint", "sign", "expm1", "log1p",
+                  "deg2rad", "rad2deg", "floor", "ceil", "trunc"]:
+            assert_equal(hasattr(csr_matrix, f), True)
+            X2 = getattr(X, f)()
+            assert_equal(X.shape, X2.shape)
+            assert_array_equal(X.indices, X2.indices)
+            assert_array_equal(X.indptr, X2.indptr)
+            assert_array_equal(X2.toarray(), getattr(np, f)(X.toarray()))
+
+    def test_unsorted_arithmetic(self):
+        data    = arange( 5 )
+        indices = array( [7, 2, 1, 5, 4] )
+        indptr  = array( [0, 3, 5] )
+        asp = csr_matrix( (data, indices, indptr), shape=(2,10) )
+        data    = arange( 6 )
+        indices = array( [8, 1, 5, 7, 2, 4] )
+        indptr  = array( [0, 2, 6] )
+        bsp = csr_matrix( (data, indices, indptr), shape=(2,10) )
+        assert_equal((asp + bsp).todense(), asp.todense() + bsp.todense())
 
 
 class TestCSC(_TestCommon, _TestGetSet, _TestSolve,
         _TestInplaceArithmetic, _TestArithmetic,
         _TestHorizSlicing, _TestVertSlicing, _TestBothSlicing,
-        _TestFancyIndexing, TestCase):
+        _TestFancyIndexing, _Test2DSlicingRegression, TestCase):
     spmatrix = csc_matrix
 
+    @dec.knownfailureif(True, "Fancy indexing is known to be broken for CSC" \
+                              " matrices")
+    def test_fancy_indexing_set(self):
+        _TestFancyIndexing.test_fancy_indexing_set(self)
+
     def test_constructor1(self):
         b = matrix([[1,0,0,0],[0,0,1,0],[0,2,0,3]],'d')
         bsp = csc_matrix(b)
@@ -1079,6 +1238,29 @@ class TestCSC(_TestCommon, _TestGetSet, _TestSolve,
         assert_array_equal(asp.indices,[1, 2, 7, 4, 5])
         assert_array_equal(asp.todense(),bsp.todense())
 
+    def test_ufuncs(self):
+        X = csc_matrix(np.arange(21).reshape(7, 3) / 21.)
+        for f in ["sin", "tan", "arcsin", "arctan", "sinh", "tanh",
+                  "arcsinh", "arctanh", "rint", "sign", "expm1", "log1p",
+                  "deg2rad", "rad2deg", "floor", "ceil", "trunc"]:
+            assert_equal(hasattr(csr_matrix, f), True)
+            X2 = getattr(X, f)()
+            assert_equal(X.shape, X2.shape)
+            assert_array_equal(X.indices, X2.indices)
+            assert_array_equal(X.indptr, X2.indptr)
+            assert_array_equal(X2.toarray(), getattr(np, f)(X.toarray()))
+
+    def test_unsorted_arithmetic(self):
+        data    = arange( 5 )
+        indices = array( [7, 2, 1, 5, 4] )
+        indptr  = array( [0, 3, 5] )
+        asp = csc_matrix( (data, indices, indptr), shape=(10,2) )
+        data    = arange( 6 )
+        indices = array( [8, 1, 5, 7, 2, 4] )
+        indptr  = array( [0, 2, 6] )
+        bsp = csc_matrix( (data, indices, indptr), shape=(10,2) )
+        assert_equal((asp + bsp).todense(), asp.todense() + bsp.todense())
+
 
 class TestDOK(_TestCommon, _TestGetSet, _TestSolve, TestCase):
     spmatrix = dok_matrix
@@ -1166,11 +1348,11 @@ class TestDOK(_TestCommon, _TestGetSet, _TestSolve, TestCase):
         # The next 6 commands should raise exceptions
         try:
             A[0,0] = range(100)
-        except TypeError:
+        except ValueError:
             caught += 1
         try:
             A[0,0] = arange(100)
-        except TypeError:
+        except ValueError:
             caught += 1
         try:
             A[0,:] = range(100)
@@ -1204,11 +1386,132 @@ class TestDOK(_TestCommon, _TestGetSet, _TestSolve, TestCase):
         c = csr_matrix(b)
         assert_equal(A.todense(), c.todense())
 
+    def test_resize(self):
+        """A couple basic tests of the resize() method.
 
+        resize(shape) resizes the array in-place.
+        """
+        a = dok_matrix((5,5))
+        a[:,0] = 1
+        a.resize((2,2))
+        expected1 = array([[1,0],[1,0]])
+        assert_array_equal(a.todense(), expected1)
+        a.resize((3,2))
+        expected2 = array([[1,0],[1,0],[0,0]])
+        assert_array_equal(a.todense(), expected2)
+
+
+    def test_ticket1160(self):
+        """Regression test for ticket #1160."""
+        a = dok_matrix((3,3))
+        a[0,0] = 0
+        # This assert would fail, because the above assignment would
+        # incorrectly call __set_item__ even though the value was 0.
+        assert_((0,0) not in a.keys(), "Unexpected entry (0,0) in keys")
+
+        # Slice assignments were also affected.
+        b = dok_matrix((3,3))
+        b[:,0] = 0
+        assert_(len(b.keys())==0, "Unexpected entries in keys")
+
+    # The following five tests are duplicates from _TestCommon, so they can be
+    # marked as knownfail for Python 2.4.  Once 2.4 is no longer supported,
+    # these duplicates can be removed again.
+
+    @dec.knownfailureif(sys.version[:3] == '2.4', "See ticket 1559")
+    def test_add_dense(self):
+        """ adding a dense matrix to a sparse matrix
+        """
+        sum1 = self.dat + self.datsp
+        assert_array_equal(sum1, 2*self.dat)
+        sum2 = self.datsp + self.dat
+        assert_array_equal(sum2, 2*self.dat)
+
+    @dec.knownfailureif(sys.version[:3] == '2.4', "See ticket 1559")
+    def test_radd(self):
+        a = self.dat.copy()
+        a[0,2] = 2.0
+        b = self.datsp
+        c = a + b
+        assert_array_equal(c,[[2,0,2,4],[6,0,2,0],[0,4,0,0]])
+
+    @dec.knownfailureif(sys.version[:3] == '2.4', "See ticket 1559")
+    def test_rsub(self):
+        assert_array_equal((self.dat - self.datsp),[[0,0,0,0],[0,0,0,0],[0,0,0,0]])
+        assert_array_equal((self.datsp - self.dat),[[0,0,0,0],[0,0,0,0],[0,0,0,0]])
+
+        A = self.spmatrix(matrix([[1,0,0,4],[-1,0,0,0],[0,8,0,-5]],'d'))
+        assert_array_equal((self.dat - A),self.dat - A.todense())
+        assert_array_equal((A - self.dat),A.todense() - self.dat)
+        assert_array_equal(A.todense() - self.datsp,A.todense() - self.dat)
+        assert_array_equal(self.datsp - A.todense(),self.dat - A.todense())
+
+    @dec.knownfailureif(sys.version[:3] == '2.4', "See ticket 1559")
+    def test_matmat_sparse(self):
+        a = matrix([[3,0,0],[0,1,0],[2,0,3.0],[2,3,0]])
+        a2 = array([[3,0,0],[0,1,0],[2,0,3.0],[2,3,0]])
+        b = matrix([[0,1],[1,0],[0,2]],'d')
+        asp = self.spmatrix(a)
+        bsp = self.spmatrix(b)
+        assert_array_almost_equal((asp*bsp).todense(), a*b)
+        assert_array_almost_equal( asp*b, a*b)
+        assert_array_almost_equal( a*bsp, a*b)
+        assert_array_almost_equal( a2*bsp, a*b)
+
+        # Now try performing cross-type multplication:
+        csp = bsp.tocsc()
+        c = b
+        assert_array_almost_equal((asp*csp).todense(), a*c)
+        assert_array_almost_equal( asp*c, a*c)
+
+        assert_array_almost_equal( a*csp, a*c)
+        assert_array_almost_equal( a2*csp, a*c)
+        csp = bsp.tocsr()
+        assert_array_almost_equal((asp*csp).todense(), a*c)
+        assert_array_almost_equal( asp*c, a*c)
+
+        assert_array_almost_equal( a*csp, a*c)
+        assert_array_almost_equal( a2*csp, a*c)
+        csp = bsp.tocoo()
+        assert_array_almost_equal((asp*csp).todense(), a*c)
+        assert_array_almost_equal( asp*c, a*c)
+
+        assert_array_almost_equal( a*csp, a*c)
+        assert_array_almost_equal( a2*csp, a*c)
+
+        # Test provided by Andy Fraser, 2006-03-26
+        L = 30
+        frac = .3
+        random.seed(0) # make runs repeatable
+        A = zeros((L,2))
+        for i in xrange(L):
+            for j in xrange(2):
+                r = random.random()
+                if r < frac:
+                    A[i,j] = r/frac
+
+        A = self.spmatrix(A)
+        B = A*A.T
+        assert_array_almost_equal(B.todense(), A.todense() * A.T.todense())
+        assert_array_almost_equal(B.todense(), A.todense() * A.todense().T)
+
+        # check dimension mismatch  2x2 times 3x2
+        A = self.spmatrix( [[1,2],[3,4]] )
+        B = self.spmatrix( [[1,2],[3,4],[5,6]] )
+        assert_raises(ValueError, A.__mul__, B)
+
+    @dec.knownfailureif(sys.version[:3] == '2.4', "See ticket 1559")
+    def test_sub_dense(self):
+        """ subtracting a dense matrix to/from a sparse matrix
+        """
+        sum1 = 3*self.dat - self.datsp
+        assert_array_equal(sum1, 2*self.dat)
+        sum2 = 3*self.datsp - self.dat
+        assert_array_equal(sum2, 2*self.dat)
 
 class TestLIL( _TestCommon, _TestHorizSlicing, _TestVertSlicing,
         _TestBothSlicing, _TestGetSet, _TestSolve,
-        _TestArithmetic, _TestInplaceArithmetic,
+        _TestArithmetic, _TestInplaceArithmetic, _TestFancyIndexing,
         TestCase):
     spmatrix = lil_matrix
 
@@ -1218,6 +1521,17 @@ class TestLIL( _TestCommon, _TestHorizSlicing, _TestVertSlicing,
     B[2,1] = 3
     B[3,0] = 10
 
+
+    @dec.knownfailureif(True, "Fancy indexing is known to be broken for LIL" \
+                              " matrices")
+    def test_fancy_indexing_set(self):
+        _TestFancyIndexing.test_fancy_indexing_set(self)
+
+    @dec.knownfailureif(True, "Fancy indexing is known to be broken for LIL" \
+                              " matrices")
+    def test_fancy_indexing_randomized(self):
+        _TestFancyIndexing.test_fancy_indexing_randomized(self)
+
     def test_dot(self):
         A = matrix(zeros((10,10)))
         A[0,3] = 10
@@ -1293,7 +1607,7 @@ class TestLIL( _TestCommon, _TestHorizSlicing, _TestVertSlicing,
         B[:2,:2] = csc_matrix(array(block))
         assert_array_equal(B.todense()[:2,:2],block)
 
-    def test_lil_sequence_assignement(self):
+    def test_lil_sequence_assignment(self):
         A = lil_matrix((4,3))
         B = eye(3,4,format='lil')
 
@@ -1306,6 +1620,16 @@ class TestLIL( _TestCommon, _TestHorizSlicing, _TestVertSlicing,
         A[2,i2] = B[i2,2]
         assert_array_equal(A.todense(),B.T.todense())
 
+        # column slice
+        A = lil_matrix((2,3))
+        A[1,1:3] = [10,20]
+        assert_array_equal(A.todense(), [[0,0,0],[0,10,20]])
+
+        # column slice
+        A = lil_matrix((3,2))
+        A[1:3,1] = [[10],[20]]
+        assert_array_equal(A.todense(), [[0,0],[0,10],[0,20]])
+
     def test_lil_iteration(self):
         row_data = [[1,2,3],[4,5,6]]
         B = lil_matrix(array(row_data))
@@ -1355,6 +1679,11 @@ class TestLIL( _TestCommon, _TestHorizSlicing, _TestVertSlicing,
                             [0,0,9],
                             [0,16,0]])
 
+    def test_lil_multiply_removal(self):
+        """Ticket #1427."""
+        a = lil_matrix(np.ones((3,3)))
+        a *= 2.
+        a[0, :] = 0
 
 
 class TestCOO(_TestCommon, TestCase):
@@ -1418,7 +1747,6 @@ class TestDIA(_TestCommon, _TestArithmetic, TestCase):
         assert_equal(dia_matrix( (data,offsets), shape=(4,4)).todense(), D)
 
 
-
 class TestBSR(_TestCommon, _TestArithmetic, _TestInplaceArithmetic, TestCase):
     spmatrix = bsr_matrix
 
